%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is coded by Hyun-Chul Choi
% For Christine Seul Ki Kim
% initiated on 2014.01.26
% updated on 2014.08.10
% updated on 2014.08.20
% updated on 2015.05.23
% last updated on 2015.07.17
% copyright@Hyun-Chul Choi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function image_analysis(image_folder_name, result_folder_name, width_length, offset, offset_reverse, sub_num, threshold, initial_effective_ratio, T1, T2, level_scale)

% fid = fopen('gt.txt','r');
% gt = fscanf(fid, '%d');
% fclose(fid);
% gt = reshape(gt, [33 length(gt)/33])';

effective_sub_num = ceil(sub_num * initial_effective_ratio);

start_template = [-1 -1 -1 -1 1 1 1];
end_template = [1 1 1 -1 -1 -1 -1];

processing_time = 0;
image_num = 0;

% figure(10);
% subplot(1,2,1);
% plot(1:7, start_template, 'o');
% axis([1 7 -1.5 1.5]);
% title('7-point step function for rising edge');
% subplot(1,2,2);
% plot(1:7, end_template, 'o');
% axis([1 7 -1.5 1.5]);
% title('7-point step function for falling edge');

mkdir(result_folder_name);

result_file_name = [result_folder_name '/result_list.txt'];
fid = fopen(result_file_name, 'wt');
fprintf(fid, 'file_name\t');
fprintf(fid, 'pixel_avg\tum_avg\t');
for j=1:sub_num;
    fprintf(fid, 'pixel%d\tum%d\t', j, j);
end
fprintf(fid, '\n');

%data_distribution = {};

file_list = dir(image_folder_name);
for i=1:length(file_list);
    if file_list(i).isdir == 1 || ~isempty(strfind(file_list(i).name, 'Thumbs.db')),
        continue;
    end
    image_file_name = [image_folder_name '/' file_list(i).name];
    result_file_name = [result_folder_name '/' file_list(i).name];
    display(['image file name: ' image_file_name]);

    image = imread(image_file_name);

    tic;
    if size(image,3) ~= 1,
        image_gray = image(:,:,1);
    else
        image_gray = image;
    end
    scale_factor = width_length / size(image_gray, 2); % calculating scale factor for unit conversion
    
    level = graythresh(image_gray);
    image_gray = im2bw(image_gray,level*level_scale);
%   image_gray_noisy = image_gray;

    se = strel('square',5);
    for j=1:2;
        image_gray = imdilate(image_gray,se);
    end
    for j=1:3;
        image_gray = imerode(image_gray,se);
    end

    width = floor(size(image_gray,2)/sub_num);
    start_index = zeros(sub_num,1);
    end_index = zeros(sub_num,1);
    height = zeros(sub_num,1);
    image_normalized_sum_noisy = zeros(size(image_gray,1),sub_num);
    image_normalized_sum = zeros(size(image_gray,1),sub_num);
    bipolar_map = zeros(size(image_gray,1),sub_num);
    for j=1:sub_num;
        sub_image_gray = image_gray(:,(j-1)*width+(1:width));
        
        image_sum = sum(sub_image_gray,2);
        image_normalized_sum_noisy(:,j) = image_sum / size(sub_image_gray,2);
%        image_normalized_sum(:,j) = smooth(image_normalized_sum_noisy(:,j),5);
        image_normalized_sum(:,j) = filter(ones(5,1), 5, image_normalized_sum_noisy(:,j));
        bipolar_map(:,j) = 2*(image_normalized_sum(:,j) >= threshold) - 1;

        start_index(j) = -1;
        end_index(j) = -1;
        height(j) = -1;
        for k=length(bipolar_map(:,j))-3-offset_reverse:-1:offset+4;
            if (end_index(j) < 0) && (end_template * bipolar_map(k-3:k+3,j) == length(end_template)),
                end_index(j) = k-1;
            elseif (end_index(j) > 0) && (start_template * bipolar_map(k-3:k+3,j) == length(start_template)),
                start_index(j) = k+1;

                height(j) = end_index(j) - start_index(j) + 1;
                break;
            end
        end
    end

%    filtered_map = filter(end_template, 1, bipolar_map);
%    filtered_map2 = filter(start_template, 1, bipolar_map);

    selected_index = find((start_index ~= -1) & (end_index ~= -1));
    % error dealing for no effective subsection
    if length(selected_index) < effective_sub_num,
        elapsed_time = toc;
        display(': error = -1');
        display([': elapsed time (ms) = ' num2str(elapsed_time * 1000)]);

        % write error data in text file
        print_error_message(fid, file_list(i).name, sub_num, -1);
        % write output image
        imwrite(image, result_file_name);
        
        continue;
    end
    
    start_index_refined = start_index(selected_index);
    end_index_refined = end_index(selected_index);
    width_refined = end_index_refined - start_index_refined;
    [~, width_index] = sort(width_refined);
    if length(width_index) < effective_sub_num,
        effective_index = width_index;
    else
        effective_index = width_index(round(length(width_index)/2) - round(effective_sub_num/2) + (1:effective_sub_num));
    end
    initial_mean_height = mean(width_refined(effective_index));
    effective_index = zeros(length(width_index),1);
    effective_index_num = 0;
    for j = 1:length(width_index);
        difference = abs(width_refined(width_index(j)) - initial_mean_height);
        if difference < initial_mean_height * T1 || difference < T2,
            effective_index_num = effective_index_num + 1;
            effective_index(effective_index_num) = width_index(j);
        end
    end
    effective_index = selected_index(effective_index(1:effective_index_num));
%   effective_index = selected_index(width_index(sub_num_el+1:length(width_index)-sub_num_el));
    if isempty(effective_index),
        elapsed_time = toc;
        display(': error = -2');
        display([': elapsed time (ms) = ' num2str(elapsed_time * 1000)]);
        
        % write error data in text file
        print_error_message(fid, file_list(i).name, sub_num, -2);
        % write output image
        imwrite(image, result_file_name);
        
        continue;
    end
    mean_start_index = mean(start_index(effective_index));
    mean_end_index = mean(end_index(effective_index));
    mean_height = mean_end_index - mean_start_index + 1;
    mean_um = mean_height * scale_factor;
    
    elapsed_time = toc;
    display([': avg. depth = ' num2str(mean_height) ' (pixel), ' num2str(mean_um) ' (um)']);
    display([': elapsed time (ms) = ' num2str(elapsed_time * 1000)]);
    
    processing_time = processing_time + elapsed_time;
    image_num = image_num + 1;

    % write output data in text file
    fprintf(fid, '%s\t', file_list(i).name);
    fprintf(fid, '%f\t%f\t', mean_height, mean_um);
    for j=1:sub_num;
        fprintf(fid, '%d\t%f\t', height(j), height(j)*scale_factor);
    end
    fprintf(fid, '\n');
%     for j=1:size(gt,1);
%         if gt(j,1) == str2num(file_list(i).name(1:strfind(file_list(i).name, '.jpg')-1));
%             gt_index = j;
%             break;
%         end
%     end
    
    % write output image
%   image(:,:,1) = image_gray*255;
    image(round(mean_start_index),:,:) = 255;
    image(round(mean_end_index),:,:) = 255;
    for j=1:sub_num;
%       image(:,j*width+1,1) = 80;
%       image(:,j*width+1,2) = 80;
%       image(:,j*width+1,3) = 80;
        if start_index(j) == -1 || end_index(j) == -1,
            continue;
        end
        image(start_index(j),(j-1)*width+(1:width),1) = 0;
        image(start_index(j),(j-1)*width+(1:width),2) = 0;
        image(start_index(j),(j-1)*width+(1:width),3) = 255;
        image(end_index(j),(j-1)*width+(1:width),1) = 0;
        image(end_index(j),(j-1)*width+(1:width),2) = 0;
        image(end_index(j),(j-1)*width+(1:width),3) = 255;
%       H = vision.TextInserter(num2str(j));
%       H.Color = [100 100 100];
%       H.FontSize = 12;
%       H.Location = [start_index(j) round((j-1)*width+1)];
%       image = step(H, image);
    end
    for k=1:length(effective_index);
        j = effective_index(k);
        image(start_index(j),(j-1)*width+(1:width),1) = 0;
        image(start_index(j),(j-1)*width+(1:width),2) = 255;
        image(start_index(j),(j-1)*width+(1:width),3) = 0;
        image(end_index(j),(j-1)*width+(1:width),1) = 0;
        image(end_index(j),(j-1)*width+(1:width),2) = 255;
        image(end_index(j),(j-1)*width+(1:width),3) = 0;
    end
    imwrite(image, result_file_name);
    
%    data_distribution{i}.data = [start_index end_index gt(gt_index,2:end)'];
%    data_distribution{i}.mean = initial_mean_height;
    
%     figure(); imshow(image_gray_noisy); %%%%
%     title('binarized image');
%     figure(); imshow(image_gray); %%%%%%%%%%
%     hold on;
%     title('binarized image after morphology operations');
%     for j=2:8:sub_num;
%         if start_index(j) == -1 || end_index(j) == -1,
%             continue;
%         end
%         plot(width*(j-1)*ones(1,size(image_normalized_sum,1)), 1:size(image_normalized_sum,1), 'm:');
%         plot(width*j*ones(1,size(image_normalized_sum,1)), 1:size(image_normalized_sum,1), 'm:');
%     end
%     hold off;
% %    figure;
%     plot_index = 1;
%     for j=2:8:sub_num;
% %        subplot(2,2,plot_index);
%         figure();
%         plot(image_normalized_sum_noisy(:,j), 'b');
%         axis([0 length(image_normalized_sum(:,j)) -0.5 1.5]);
%         title([image_file_name ' sub' num2str(j)]);
% 
%         figure();
%         plot(image_normalized_sum(:,j), 'b');
%         hold on;
%         plot([0 length(image_normalized_sum(:,j))], [threshold threshold], 'r'); 
%         plot([start_index(j) start_index(j)], [-0.5 1.5], 'k:');
%         plot([end_index(j) end_index(j)], [-0.5 1.5], 'k:');
%         hold off;
%         axis([0 length(image_normalized_sum(:,j)) -0.5 1.5]);
%         title([image_file_name ' sub' num2str(j)]);
%         xlabel('vertical pixel index');
%         ylabel('value');
%         legend('normalized row sum', 'threshold');
% 
%         figure();
%         plot(bipolar_map(:,j), 'b');
%         axis([0 length(image_normalized_sum(:,j)) -1.5 1.5]);
%         title([image_file_name ' sub' num2str(j)]);
% 
%         figure();
%         plot(filtered_map(:,j), 'b');
%         axis([0 length(filtered_map(:,j)) -8 8]);
%         title([image_file_name ' sub' num2str(j)]);
% 
%         figure();
%         plot(filtered_map2(:,j), 'b');
%         axis([0 length(filtered_map2(:,j)) -8 8]);
%         title([image_file_name ' sub' num2str(j)]);
%         
%         plot_index = plot_index + 1;
%     end

end

fclose(fid);

display(['avg. processing time (ms) = ' num2str(processing_time/image_num * 1000)]);

% plot data distribution
%save('distribution.mat', 'data_distribution', 'file_list');
end